use sysinfo::System;
use std::fmt::Display;

fn main() {
    get_sys_info();
}

fn printify<A: AsRef<str>, B: Display>(a: A, b: B) {
    println!("{:30}{}", a.as_ref(), b);
}

fn get_sys_info() {
    let bytes_per_gib: u64 = 2u64.pow(30);

    // Please note that we use "new_all" to ensure that all list of
    // components, network interfaces, disks and users are already
    // filled!
    let mut sys = System::new_all();

    // First we update all information of our `System` struct.
    sys.refresh_all();

    // Store all system information:
    let hostname = System::host_name().unwrap();

    let os_name = System::name().unwrap();
    let os_version = System::os_version().unwrap();

    let uptime_hours = System::uptime() / 3600;
    let uptime_minutes = (System::uptime() % 3600) / 60;
    let uptime = format!("{} hour(s) and {} minute(s)", uptime_hours, uptime_minutes);

    let kernel_version = System::kernel_version().unwrap();

    let cpu_model = sys.cpus()[0].brand();
    let cpu_cores = sys.physical_core_count().unwrap();
    let cpu_threads = sys.cpus().len();
    let cpu_topology = format!("{} cores {} threads", cpu_cores, cpu_threads);
    let cpu_architecture = System::cpu_arch().unwrap();

    let total_memory = sys.total_memory() as f64 / bytes_per_gib as f64;
    let memory = format!("{:.2} GB", total_memory);

    // Print system information:
    printify("Hostname:", hostname);
    printify("OS Name:", os_name);
    printify("OS Version:", os_version);
    printify("Uptime:", uptime);
    printify("Kernel Version:", kernel_version);

    // Print CPU information:
    printify("CPU Model:", cpu_model);
    printify("CPU Topology:", cpu_topology);
    printify("CPU Architecture:", cpu_architecture);

    // Print memory information:
    printify("Total Memory:", memory);
}